@@ -9,7 +9,7 @@ use Carp;
 use Config;
 use Math::DCT 'dct2d';
 
-our $VERSION = '0.0_2';
+our $VERSION = '0.1_1';
 
 =head1 NAME
 
@@ -19,8 +19,9 @@ Image::PHash - Fast perceptual image hashing (DCT-based pHash)
 
   use Image::PHash;
 
-  # Load an image and resize to the default 32x32 that will be the base for calculating the DCT matrix
-  my $iph = Image::PHash->new($image_file, 'Imlib2');
+  # Load an image and prepare to hash.
+  # Will try to find an image library to load and resize the image to 32x32, ready for the DCT step
+  my $iph = Image::PHash->new($image_file);
 
   # Calculate the perceptual hash (top 8x8 of the DCT) - 64 bits, 16 hex chars
   my $p = $iph->pHash(); # implies settings geometry => '8x8', method => 'average'
@@ -28,7 +29,7 @@ Image::PHash - Fast perceptual image hashing (DCT-based pHash)
   # Alternative, better performing, 64 bit hash (64 upper-left most DCT values)
   my $p = $iph->pHash(geometry => 64); # this is actually recommended over the default hash
 
-  # Different method for the bitmask for lowest false-negative rate
+  # Alternative method for creating the bitmask with lower false-negative rate
   my $p = $iph->pHash(geometry => 64, method => 'log');
 
   # Calculate a pHash with the upper left half of the upper-left-most 7x7 of the DCT - 27 bits, 7 hex chars
@@ -52,13 +53,13 @@ Image::PHash allows you to calculate the (DCT-based) perceptual hash (pHash) of
 
 The constructor and general structure is based on L<Image::Hash> - keeping usage quite
 similar, but the pHash algorithm is rewritten from scratch as the L<Image::Hash>
-implementation was seriously flawed (and slow). Apart from fixes for GD, Imager,
-ImageMagick resizing, L<Image::Imlib2> support is added along with some unique features
-like reduced hashes for indexing, options to deal with image mirroring, alternative
+implementation was flawed (and slow). Apart from fixes/tweaks for GD, Imager, ImageMagick
+resizing, L<Image::Imlib2> support is added along with some unique features like
+reduced hashes for indexing, options to deal with image mirroring, alternative
 bitmap methods.
 
-A fast DCT module made for the specific purpose is used (L<Math::DCT> - over 10x
-faster than the pHash.org C++ implementation).
+A fast DCT XS module made specifically to serve this hashing module is used: L<Math::DCT>.
+Depending on your setup, it should be 15x or so faster than pHash.org
 
 =head1 CONSTRUCTOR METHOD
 
@@ -172,9 +173,10 @@ the entire NxN matrix for C<geometry='NxN'>, or use the next X lowest frequency
 perhaps a bit better (lower) false negatives.
 
 =item * C<log> : A special logarithmic average is calculated as the threshold, giving
-the lowest collision rate of all the methods. It will usually have a bit increased false
-negative chance compared to C<average> or perhaps even C<diff> - but should still be lower
-than C<median>. Quite close to C<average_x>, but also applies to non-reduced hashes.
+the lowest collision rate (a tie with <median> or better). It will usually have a bit
+increased false negative chance compared to C<average> or perhaps even C<diff> - but
+should still be lower than C<median>. Quite close to C<average_x>, but also applies to
+non-reduced hashes.
 
 =item * C<diff> : The difference between each DCT value is taken as the bitmask. It seems
 to fall between C<average> and C<median> in most tests, both in false negative/collision
@@ -238,7 +240,7 @@ to pHash7, but will produce more matches when used as an index, along with many
 positives.
 
 Note that neither reduced version is appropriate to use as an image comparison hash by itself
-(too many false positives), and they are chosen to be complimentary, so when used in conjuction
+(too many false positives), and they are chosen to be complimentary, so when used in conjunction
 for either indexing or verification, their performance increases considerably.
 
 =head1 HELPER METHODS
@@ -264,7 +266,7 @@ Will return the full 32x32 DCT as an arrayref of floats.
   );
 
 Will return a print-friendly reduced size bitmask matrix as a string. Basically a
-string with rows/columns of the 1s and 0s you would get from calling C<$iph->pHash()>
+string with rows/columns of the 1s and 0s you would get from calling C<$iph-E<gt>pHash()>
 with the same parameters.
 
 =head1 HELPER FUNCTIONS
@@ -282,7 +284,8 @@ Will convert a bit value string to a hex string.
 Will calculate the bit difference of two hex string hashes (their Hamming distance
 of their bit stream form). On 64 bit systems (checking C<$Config{ivsize}>) it will
 actually call C<_diff64> which can calculate the difference of up to 64bit hashes in
-a single operation (using C<%064b>).
+a single operation (using C<%064b>). You can call C<_diff64> directly if you prefer
+in that scenario.
 
 =head1 NOTES
 
@@ -297,7 +300,7 @@ So, most of the processing time is spent on loading the image, resizing, extract
 pixel values, removing color, all of which depend on the specific image module. On an
 Apple M1, hashing 800x600 jpg images was measured at 131 h/s with L<Image::Magick>,
 208 h/s with L<Imager>, 241 h/s with L<GD>, 547 h/s with L<Image::Imlib2>.
-Higher resulutions make the process slower as you could expect. Since all images will
+Higher resolutions make the process slower as you could expect. Since all images will
 be resized to 32x32 in the end, the fastest hashing performance would be if you loaded
 32x32 thumbnails. In that case, the performance of the libraries in the same order for
 the resized imageset were: 659 h/s, 664 h/s, 1883 h/s, 2296 h/s. It is clear that
@@ -341,19 +344,19 @@ Despite it not doing a full DCT it was really slow (over 80x slower than the XS
 so switching to L<Image::PHash> will give you "correct" hashes at a significant speed
 increase, along with several extra features.
 
-=head2 L<Image::PHash> vs L<phash.org>
+=head2 L<Image::PHash> vs L<pHash.org>
 
 Apart from the significant speed advantage of Image::PHash noted above, there are a couple
-of important differences, in that phash.org will apply a 7x7 mean filter to the image
+of important differences, in that pHash.org will apply a 7x7 mean filter to the image
 before the resize and the conversion to bits is always done with the median method. This
 seems to keep false positives quite low, but its false negatives are higher. Since with
 Image::PHash you can get even better hashes with, for example, C<geometry=64> and you can
 combine them with C<method='average_x'> or C<method='log'>, you will get even lower false
-positive rate than phash.org, but with less false negatives as well. Feel free to share
+positive rate than pHash.org, but with less false negatives as well. Feel free to share
 your own comparisons with the author if in doubt.
 
-Note that the differences you are to use as a threshold for Image::PHash and phash.org are
-quite different - phash.org will give about 50% greater diffs on average (e.g. where I would
+Note that the differences you are to use as a threshold for Image::PHash and pHash.org are
+quite different - pHash.org will give about 50% greater diffs on average (e.g. where I would
 use 7 for the former, 11 would be the equivalent for the latter).
 
 =head2 Selecting a C<diff> threshold
@@ -370,6 +373,8 @@ respectively are useful for lookups (and still with lots of false positives as n
 
 =cut
 
+my $_64bit;
+
 sub pHash7 {
     my ($self, %opt) = @_;
     return $self->pHash(%opt, geometry => '7x7', reduce => 1);
@@ -492,10 +497,15 @@ sub b2h {
     return $hex;
 }
 
+sub _is_64bit {
+    $_64bit //= $Config{ivsize} >= 8;
+    return $_64bit;
+}
+
 # Difference in bits between two hex strings
 # About 30% slower than using %064b directly, but this is portable to 32 bits
 sub diff {
-    return _diff64(@_) if length($_[0]) <= 16 && $Config{ivsize} >= 8;
+    return _diff64(@_) if length($_[0]) <= 16 && ($_64bit || &_is_64bit);
     my $diff;
     for (my $i = 0; $i < length($_[0]); $i += 8) {
         my $d =
@@ -666,13 +676,13 @@ sub dctdump {
 }
 
 sub printbitmatrix {
-    my ($self, %opt) = @_;
-
+    my $self  = shift;
+    my %opt   = $self->_validate_options(@_);
     my @array = $self->pHash(%opt);
     my $sep   = $opt{separator} || '';
     my $fill  = $opt{filler} || ' ';
 
-    my ($xs, $ys) = split(/x/, $opt{geometry} || '8x8');
+    my ($xs, $ys) = split(/x/, $opt{geometry});
     my $str = '';
     if ($ys) {
         for (my $i = 0; $i < $xs; $i++) {
@@ -808,8 +818,8 @@ sub _dct_mirrorproof {
 
 sub _apply_average {
     my $array   = shift;
-    my $skip    = shift || 0;
-    my $extra   = shift || [];
+    my $skip    = shift;
+    my $extra   = shift;
     my $average = 0;
     $average += $array->[$_] foreach $skip .. $#$array;
     $average += $_           foreach @$extra;
@@ -819,7 +829,7 @@ sub _apply_average {
 
 sub _apply_log_average {
     my $array   = shift;
-    my $skip    = shift || 0;
+    my $skip    = shift;
     my $exp     = 1 + (scalar(@$array)**(1 / 6));
     my $average = 0;
     foreach ($skip .. $#$array) {
@@ -827,7 +837,7 @@ sub _apply_log_average {
         $average += $array->[$_] < 0 ? -$add : $add;
     }
     $average /= scalar(@$array) - $skip;
-    my $thres = $average**$exp;
+    my $thres = abs($average)**$exp;
     $thres *= -1 if $average < 0;
     $array->[$_] = $array->[$_] >= $thres ? '1' : '0' foreach 0 .. $#$array;
 }
@@ -844,8 +854,8 @@ sub _apply_diff {
 
 sub _apply_median {
     my $array = shift;
-    my $skip  = shift || 0;
-    my $extra = shift || [];
+    my $skip  = shift;
+    my $extra = shift;
     my @vals  = sort {$a <=> $b} @$array, @$extra;
     my $len   = scalar @vals - $skip;
     my $thresh =
